--SavedVariables Setup
local DejaViewClassic, private = ...

private.defaults = {
	-- Holds all default settings hierarchically.
}

private.defaults.dvdefaults = {
	-- Holds all default settings for DejaViewClassic, specifically.
}

local loader = CreateFrame("Frame")
	loader:RegisterEvent("ADDON_LOADED")
	loader:SetScript("OnEvent", function(self, addon)
		if addon ~= DejaViewClassic then 
			local function initDB(db, defaults)
				if type(db) ~= "table" then db = {} end
				if type(defaults) ~= "table" then return db end
				for k, v in pairs(defaults) do
					if type(v) == "table" then
						db[k] = initDB(db[k], v)
					elseif type(v) ~= type(db[k]) then
						db[k] = v
					end
				end
			return db
			end

		DejaViewClassicDBPC = initDB(DejaViewClassicDBPC, private.defaults)
		private.db = DejaViewClassicDBPC -- add this
		self:UnregisterEvent("ADDON_LOADED")
		-- Don't place any frames here
		end
	end)

DejaViewClassic = {};

local _, private = ...
	private.defaults.dvdefaults.optpanelDefaults = {
		point = "RIGHT", 
		relativeTo = "UIParent", 
		relativePoint = "RIGHT", 
		xOffset = 0, 
		yOffset = 0,
	}	
	
DejaViewClassic.panel = CreateFrame( "Frame", "DejaViewClassicPanel", InterfaceOptionsFramePanelContainer );
-- Register in the Interface Addon Options GUI
-- Set the name for the Category for the Options Panel
DejaViewClassic.panel.name = "DejaViewClassic";
-- Add the panel to the Interface Options
InterfaceOptions_AddCategory(DejaViewClassic.panel);

-- Make a child panel
-- DejaViewClassic.childpanel = CreateFrame( "Frame", "DejaViewClassicChild", DejaViewClassic.panel);
-- DejaViewClassic.childpanel.name = "MyChild";
-- Specify childness of this panel (this puts it under the little red [+], instead of giving it a normal AddOn category)
-- DejaViewClassic.childpanel.parent = DejaViewClassic.panel.name;
-- Add the child to the Interface Options
-- InterfaceOptions_AddCategory(DejaViewClassic.childpanel);

-- Panel Title
local dvtitle=CreateFrame("Frame", "dvtitle", DejaViewClassicPanel)
	dvtitle:SetPoint("TOPLEFT", 5, -5)
	dvtitle:SetScale(2.0)
	dvtitle:SetWidth(150)
	dvtitle:SetHeight(50)
	dvtitle:Show()

local dvtitleFS = dvtitle:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	dvtitleFS:SetText('|cff00c0ffDejaViewClassic|r')
	dvtitleFS:SetPoint("TOPLEFT", 0, 0)
	dvtitleFS:SetFont("Fonts\\FRIZQT__.TTF", 10)
	
dvresetcheck = CreateFrame("Button", "DVResetButton", DejaViewClassicPanel, "UIPanelButtonTemplate")
	dvresetcheck:ClearAllPoints()
	dvresetcheck:SetPoint("BOTTOMLEFT", 5, 5)
	dvresetcheck:SetScale(1.25)
	dvresetcheck:SetWidth(125)
	dvresetcheck:SetHeight(30)
	dvresetcheck.tooltipText = 'Reset DejaViewClassic options to default settings.' --Creates a tooltip on mouseover.
	_G[dvresetcheck:GetName() .. "Text"]:SetText("Reset to Default")
	
	dvresetcheck:SetScript("OnClick", function (self, button, down)
 		private.db.dvdefaults = private.defaults.dvdefaults;
		ReloadUI();
end)

 dvlockcheck = CreateFrame("CheckButton", "DejaViewClassicLockAll", DejaViewClassicPanel, "InterfaceOptionsCheckButtonTemplate")
	dvlockcheck:ClearAllPoints()
	dvlockcheck:SetPoint("TOPLEFT", 25, -35)
	dvlockcheck:SetScale(1.25)
	_G[dvlockcheck:GetName() .. "Text"]:SetText("Lock all frames")
	dvlockcheck.tooltipText = 'Checked locks all DejaViewClassic frames. Unchecked unlocks all DejaViewClassic frames.' --Creates a tooltip on mouseover.
	dvlockcheck:SetChecked(true)
	
	dvlockcheckframe=CreateFrame("Frame", "dvlockcheckframe", UIParent)
	dvlockcheckframe:SetClampedToScreen(1)
	dvlockcheckframe:SetFrameStrata(BACKGROUND)
	dvlockcheckframe:SetFrameLevel("1")
	dvlockcheckframe:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT")
	dvlockcheckframe:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT")
	dvlockcheckframe:Hide()
	
-- Lock/Unlock Frames
	dvlockcheck:SetScript("OnClick", function(self,button,down) 
		if self:GetChecked(true) then
			dvlockcheckframe:Hide()
		else
			dvlockcheckframe:Show()
		end
	end)
	
--Open Categories Fix
do
	local function get_panel_name(panel)
		local tp = type(panel)
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		if tp == "string" then
			for i = 1, #cat do
				local p = cat[i]
				if p.name == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel
					end
				end
			end
		elseif tp == "table" then
			for i = 1, #cat do
				local p = cat[i]
				if p == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel.name
					end
				end
			end
		end
	end

	local function InterfaceOptionsFrame_OpenToCategory_Fix(panel)
		if doNotRun or InCombatLockdown() then return end
		local panelName = get_panel_name(panel)
		if not panelName then return end -- if its not part of our list return early
		local noncollapsedHeaders = {}
		local shownpanels = 0
		local mypanel
		local t = {}
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		for i = 1, #cat do
			local panel = cat[i]
			if not panel.parent or noncollapsedHeaders[panel.parent] then
				if panel.name == panelName then
					panel.collapsed = true
					t.element = panel
					InterfaceOptionsListButton_ToggleSubCategories(t)
					noncollapsedHeaders[panel.name] = true
					mypanel = shownpanels + 1
				end
				if not panel.collapsed then
					noncollapsedHeaders[panel.name] = true
				end
				shownpanels = shownpanels + 1
			end
		end
		local Smin, Smax = InterfaceOptionsFrameAddOnsListScrollBar:GetMinMaxValues()
		if shownpanels > 15 and Smin < Smax then 
		  local val = (Smax/(shownpanels-15))*(mypanel-2)
		  InterfaceOptionsFrameAddOnsListScrollBar:SetValue(val)
		end
		doNotRun = true
		InterfaceOptionsFrame_OpenToCategory(panel)
		doNotRun = false
	end

	hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", InterfaceOptionsFrame_OpenToCategory_Fix)
end

--DV Slash Setup
local RegisteredEvents = {};
local dvslash = CreateFrame("Frame", "DejaViewClassicSlash", UIParent)

dvslash:SetScript("OnEvent", function (self, event, ...) 
	if (RegisteredEvents[event]) then 
	return RegisteredEvents[event](self, event, ...) 
	end
end)

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "DejaViewClassic") then
		SLASH_DEJAVIEWCLASSIC1 = '/dvc'
		SlashCmdList["DejaViewClassic"] = function (msg, editbox)
			DejaViewClassic.SlashCmdHandler(msg, editbox)	
		end
		DEFAULT_CHAT_FRAME:AddMessage("DejaViewClassic loaded successfully. Type /dvc for usage",0,192,255)
	end
end

for k, v in pairs(RegisteredEvents) do
	dvslash:RegisterEvent(k)
end

function DejaViewClassic.ShowHelp()
	DEFAULT_CHAT_FRAME:AddMessage("DejaViewClassic Slash commands (/dvc):")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc lock: Locks all DejaViewClassic movable frames.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc unlock: Unlocks all DejaViewClassic movable frames.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc config: Open the DejaViewClassic addon config menu.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc reset:  Resets DejaViewClassic frames to default positions.")
end

function DejaViewClassic.SetConfigToDefaults()
	DEFAULT_CHAT_FRAME:AddMessage("Resetting config to defaults")
	DejaViewClassicDBPC = DefaultConfig
	RELOADUI()
end

function DejaViewClassic.GetConfigValue(key)
	return DejaViewClassicDBPC[key]
end

function DejaViewClassic.PrintPerformanceData()
	UpdateAddOnMemoryUsage()
	local mem = GetAddOnMemoryUsage("DejaViewClassic")
	DEFAULT_CHAT_FRAME:AddMessage("DejaViewClassic is currently using " .. mem .. " kbytes of memory")
	collectgarbage(collect)
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage("DejaViewClassic")
	DEFAULT_CHAT_FRAME:AddMessage("DejaViewClassic is currently using " .. mem .. " kbytes of memory after garbage collection")
end

function DejaViewClassic.SlashCmdHandler(msg, editbox)
	--DEFAULT_CHAT_FRAME:AddMessage("command is " .. msg .. "\n")
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("DejaViewClassic");
	elseif (string.lower(msg) == "dumpconfig") then
		DEFAULT_CHAT_FRAME:AddMessage("With defaults")
		for k,v in pairs(DVDefaultConfig) do
			DEFAULT_CHAT_FRAME:AddMessage(k,DejaViewClassic.GetConfigValue(k))
		end
		DEFAULT_CHAT_FRAME:AddMessage("Direct table")
		for k,v in pairs(DVDefaultConfig) do
			DEFAULT_CHAT_FRAME:AddMessage(k,v)
		end
	elseif (string.lower(msg) == "lock") then
		dvlockcheckframe:Hide()
		dvlockcheck:SetChecked(true)
	elseif (string.lower(msg) == "unlock") then
		dvlockcheckframe:Show()
		dvlockcheck:SetChecked(false)
	elseif (string.lower(msg) == "reset") then
		DejaViewClassicDBPC = private.defaults;
		ReloadUI();
	elseif (string.lower(msg) == "perf") then
		DejaViewClassic.PrintPerformanceData()
	else
		DejaViewClassic.ShowHelp()
	end
end
	SlashCmdList["DEJAVIEWCLASSIC"] = DejaViewClassic.SlashCmdHandler;